package com.qmth.sop.server.api;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.sop.business.bean.result.EditResult;
import com.qmth.sop.business.service.TBCrmDetailService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.ProductTypeEnum;
import com.qmth.sop.common.lock.MemoryLock;
import com.qmth.sop.common.util.Result;
import com.qmth.sop.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Map;

/**
 * @Description: 派单明细表
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2024/5/14
 */
@Api(tags = "派单明细表 Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_CRM_DETAIL)
public class TBCrmDetailController {

    private final static Logger log = LoggerFactory.getLogger(TBCrmDetailController.class);

    @Resource
    MemoryLock memoryLock;

    @Resource
    TBCrmDetailService tbCrmDetailService;

    @ApiOperation(value = "导入派单明细")
    @RequestMapping(value = "/sop/import", method = RequestMethod.POST)
    @ApiResponses({ @ApiResponse(code = 200, message = "导入成功", response = EditResult.class) })
    public Result sopImport(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file,
            @ApiParam(value = "派单号", required = true) @RequestParam String crmNo,
            @ApiParam(value = "过程变量", required = true) @RequestParam String processVar,
            @ApiParam(value = "类型", required = true) @RequestParam ProductTypeEnum type) throws IOException {
        boolean lock = memoryLock.lock(SystemConstant.LOCK_CRM_DETAIL_SOP_PREFIX + crmNo, crmNo,
                SystemConstant.LOCK_CRM_DETAIL_SOP_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在导入数据，请稍候再试！");
        }
        Map<String, String> map = null;
        try {
            map = tbCrmDetailService.sopExcelImport(file, crmNo, processVar, type);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            memoryLock.unlock(SystemConstant.LOCK_CRM_DETAIL_SOP_PREFIX + crmNo);
        }
        return ResultUtil.ok(map);
    }
}
